#ifndef STORAGE_HPP
#define STORAGE_HPP

#ifdef ESP8266
#include "EEPROM.h"
#else
#include "Preferences.h"
#endif
#include "Utils.hpp"

namespace fw
{
    /**
     * @brief The Storage will handle Firewall rules in EEPROM
     *
     */
    class Storage
    {
    public:
        /**
         * @brief Construct a new Storage object
         *
         */
        Storage();

        /**
         * @brief Destroy the Storage object
         *
         */
        ~Storage();

    private:
#ifdef ESP8266
        uint8_t max_rules;
        uint16_t eeprom_size;
        uint16_t eeprom_amount_of_rules;
        uint16_t eeprom_rules_head;

        uint16_t eeprom_rule_position(uint8_t key);
#else
        Preferences memory;
#endif

    protected:
        /**
         * @brief retrieve the current amount of Firewall rules in the Storage
         *
         * @return uint8_t
         */
        uint8_t retrieve_amount_of_rules();

        /**
         * @brief store a new amount of Firewall rules in the Storage
         *
         * @param new_amount
         */
        void store_amount_of_rules(const uint8_t new_amount);

        /**
         * @brief retrieve a Firewall rule from Storage
         *
         * @param key
         * @return firewall_rule_t*
         */
        firewall_rule_t *retrieve_firewall_rule(const uint8_t key);

        /**
         * @brief store all Firewall rules in Storage
         *
         * @param rule_head
         */
        void store_all_firewall_rules(firewall_rule_t *rule_head);

        /**
         * @brief store Firewall rule in Storage
         *
         * @param rule_ptr
         */
        void store_firewall_rule(firewall_rule_t *rule_ptr);
    };
}

#endif
